@@ -1,14 +1,14 @@
-# Copyrights 2006-2015 by [Mark Overmeer].
+# Copyrights 2006-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 use warnings;
 use strict;
 no warnings 'recursion';
 
 package XML::Compile::Schema::BuiltInTypes;
 use vars '$VERSION';
-$VERSION = '1.51';
+$VERSION = '1.53';
 
 use base 'Exporter';
 
@@ -282,7 +282,7 @@ sub numcheck($)
        | NaN #x
 }
 
-$builtin_types{precissionDecimal} =
+$builtin_types{precisionDecimal} =
 $builtin_types{float}  =
 $builtin_types{double} =
  { parse   => \&str2num
@@ -560,6 +560,7 @@ $builtin_types{NMTOKEN} =
 
 $builtin_types{NMTOKENS} =
  { parse   => sub { [ split ' ', shift ] }
+ , check   => sub { $_[0] =~ /\S/ }
  , format  => sub { my $v = shift; ref $v eq 'ARRAY' ? join(' ',@$v) : $v }
  , example => 'nmtokens'
  , is_list => 1
