@@ -1,14 +1,14 @@
-# Copyrights 2006-2015 by [Mark Overmeer].
+# Copyrights 2006-2016 by [Mark Overmeer].
 #  For other contributors see ChangeLog.
 # See the manual pages for details on the licensing terms.
-# Pod stripped from pm file by OODoc 2.01.
+# Pod stripped from pm file by OODoc 2.02.
 
 use warnings;
 use strict;
 
 package XML::Compile::Schema::NameSpaces;
 use vars '$VERSION';
-$VERSION = '1.51';
+$VERSION = '1.53';
 
 
 use Log::Report 'xml-compile', syntax => 'SHORT';
@@ -143,7 +143,7 @@ sub doesExtend($$)
         }
     }
     else
-    {   # build-in
+    {   # built-in
         my ($ns, $local) = unpack_type $ext;
         $ns eq SCHEMA2001 && $builtin_types{$local}
             or error __x"cannot find {type} as simpleType or complexType"
